var path = require("path");
var child_process = require('child_process');
var url = require('url');

var dest = path.join(process.env["programfiles(x86)"], "addgears");
var installerName = "installer.exe";
var request = require('request');
var fs = require('fs');
var logger = require('./logger').common;


var json;
try{
    json = require("../addgears.config.js");
    logger.info("using options from addgears.config.js");
}catch(e){
    json = require("../package.json");
    logger.info("using options from package.json");
}

var options = json.addgears;


function download(url, dest, cb) {
    logger.info("downloading:" + url);
    var file = fs.createWriteStream(dest);
    request
    .get(url)
    .on('error', function(err) {
        logger.info(err)
    })
    .pipe(file)

    file.on('finish', function() {
      file.close(cb);
    });  
}


function install(installerName, dest, cb) {
    logger.info("installing addgears to:" + dest);
    var child = child_process.spawn(installerName, ["/S"]);

    child.stdout.on('data', function (data) {
        logger.info('stdout: ' + data);
    });

    child.stderr.on('data', function (data) {
        logger.info('stderr: ' + data);
    });

    child.on('exit', function (code) {
        if (code !== 0) {
            logger.info('process exited with code ' + code);
        }
        var exists = fs.existsSync(dest);

        if(code===0 && exists){
            logger.info('installation finished successfully');    
        }

        if(!exists){
            logger.info('could not find installetion directory:' + dest);    
        }

        if(!options.keepInstaller && code===0 && exists){
            fs.unlinkSync(installerName);
            logger.info("installer removed");
        }

        if(cb){
            cb(code===0 && exists);
        }
    });
}

function start(options) {
    if(!options){
        options = {};
    }
    var bin = options.path;

    if(!bin){
        bin = path.join(dest, "bin/nw.exe")
    }
    var nw = options.nw || path.join(path.dirname(bin), '../app.nw');

    var command = [
        nw
    ];

    if(options.command){
        command = command.concat(options.command);
    }

    logger.info("starting addgears:" + bin + " " + command.join(" "));
    var child = child_process.spawn(bin, command);

    child.stdout.on('data', function (data) {
        logger.info('stdout: ' + data);
    });

    child.stderr.on('data', function (data) {
        logger.info('stderr: ' + data);
    });

    child.on('exit', function (code) {
        if (code !== 0) {
            logger.error('process exited with code ' + code);
        }

        if(options.callback){
            options.callback(code===0);
        }       
    });
}


function uninstall(dest, cb) {
    logger.info("uninstalling addgears from:" + dest);
    var child = child_process.spawn(path.join(dest, "uninstall.exe"), ["/S"]);

    child.stdout.on('data', function (data) {
        logger.info('stdout: ' + data);
    });

    child.stderr.on('data', function (data) {
        logger.info('stderr: ' + data);
    });

    child.on('exit', function (code) {
        if (code !== 0) {
            logger.error('process exited with code ' + code);
        }
        var exists = fs.existsSync(dest);

        if(code===0){
            logger.info('uninstallation finished successfully');    
        }

        if(cb){
            cb(code===0);
        }       
    });
}

function copyTo(from, to){
    if(!from){
        return to;
    }
    for(var key in from){
        if(from.hasOwnProperty(key)){       
            to[key] = from[key];
        }
    }
    return to;
}

function rmDirSync(fs, dirPath) {
    var files = fs.readdirSync(dirPath);

    if (files.length > 0) {
        for (var i = 0; i < files.length; i++) {
            var filePath = dirPath + '/' + files[i];
            if (fs.statSync(filePath).isFile()) {
                fs.unlinkSync(filePath);
            } else {
                rmDirSync(fs, filePath);
            }
        }
    }
    fs.rmdirSync(dirPath);
}

function copyDirSyncRecursive(sourceDir, newDirLocation, opts) {
    opts = opts || {};


    /*  Create the directory where all our junk is moving to; read the mode of the source directory and mirror it */
    var checkDir = fs.statSync(sourceDir);
    try {
        fs.mkdirSync(newDirLocation, checkDir.mode);
    } catch (e) {
        //if the directory already exists, that's okay
        if (e.code !== 'EEXIST') throw e;
    }

    var files = fs.readdirSync(sourceDir);
    var preserveTimestamps = opts.preserveTimestamps === true;

    for(var i = 0; i < files.length; i++) {


        var currFile = fs.lstatSync(path.join(sourceDir, files[i]));

        var fCopyFile = function(srcFile, destFile) {
            var contents = fs.readFileSync(srcFile);
            fs.writeFileSync(destFile, contents);
            var stat =  fs.lstatSync(srcFile);
            fs.chmodSync(destFile, stat.mode);
            if (preserveTimestamps) {
                fs.utimesSync(destFile, stat.atime, stat.mtime)
            }
        };

        if(currFile.isDirectory()) {
            /*  recursion this thing right on back. */
            copyDirSyncRecursive(path.join(sourceDir, files[i]), path.join(newDirLocation, files[i]), opts);
        }
        else {
            /*  At this point, we've hit a file actually worth copying... so copy it on over. */
            fCopyFile(path.join(sourceDir, files[i]), path.join(newDirLocation, files[i]));
        }
    }
}

module.exports = {
    options:function(opt){
        return copyTo(opt, copyTo(options, {}));
    },
    rmDirSync:function (dirPath) {
        rmDirSync(fs, dirPath);
    },
    copyDirSyncRecursive:copyDirSyncRecursive,    
    installerName:installerName,
    destination:dest,
    install:install,
    uninstall:uninstall,
    download:download,
    copyTo:copyTo,
    start:start    
}
